"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertAccountNode = exports.isAccountNode = exports.AccountNode = void 0;
const utils_1 = require("../utils");
const TypeNode_1 = require("./TypeNode");
const TypeStructNode_1 = require("./TypeStructNode");
const TypePublicKeyNode_1 = require("./TypePublicKeyNode");
const TypeDefinedLinkNode_1 = require("./TypeDefinedLinkNode");
class AccountNode {
    constructor(metadata, type) {
        this.nodeClass = 'AccountNode';
        this.metadata = {
            ...metadata,
            name: (0, utils_1.mainCase)(metadata.name),
            seeds: metadata.seeds.map((seed) => 'name' in seed ? { ...seed, name: (0, utils_1.mainCase)(seed.name) } : seed),
        };
        this.type = type;
    }
    static fromIdl(idl) {
        var _a, _b, _c, _d, _e;
        const name = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
        const idlStruct = (_b = idl.type) !== null && _b !== void 0 ? _b : { kind: 'struct', fields: [] };
        const type = (0, TypeNode_1.createTypeNodeFromIdl)({ name, ...idlStruct });
        (0, TypeStructNode_1.assertTypeStructNode)(type);
        const seeds = ((_c = idl.seeds) !== null && _c !== void 0 ? _c : []).map((seed) => {
            if (seed.kind === 'variable') {
                return { ...seed, type: (0, TypeNode_1.createTypeNodeFromIdl)(seed.type) };
            }
            return seed;
        });
        const metadata = {
            name,
            idlName: name,
            docs: (_d = idl.docs) !== null && _d !== void 0 ? _d : [],
            internal: false,
            size: (_e = idl.size) !== null && _e !== void 0 ? _e : null,
            seeds,
            discriminator: null,
            gpaFields: [],
        };
        return new AccountNode(metadata, type);
    }
    accept(visitor) {
        return visitor.visitAccount(this);
    }
    get name() {
        return this.metadata.name;
    }
    get docs() {
        return this.metadata.docs;
    }
    get isLinked() {
        return (0, TypeDefinedLinkNode_1.isTypeDefinedLinkNode)(this.type);
    }
    get variableSeeds() {
        return this.metadata.seeds.filter((seed) => seed.kind === 'variable');
    }
    get hasVariableSeeds() {
        return this.variableSeeds.length > 0;
    }
    get instructionAccountDefaultSeeds() {
        return this.metadata.seeds.reduce((acc, seed) => {
            if (seed.kind !== 'variable')
                return acc;
            if ((0, TypePublicKeyNode_1.isTypePublicKeyNode)(seed.type)) {
                acc[seed.name] = { kind: 'account', name: seed.name };
            }
            else {
                acc[seed.name] = { kind: 'arg', name: seed.name };
            }
            return acc;
        }, {});
    }
}
exports.AccountNode = AccountNode;
function isAccountNode(node) {
    return !!node && node.nodeClass === 'AccountNode';
}
exports.isAccountNode = isAccountNode;
function assertAccountNode(node) {
    var _a;
    if (!isAccountNode(node)) {
        throw new Error(`Expected AccountNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertAccountNode = assertAccountNode;
//# sourceMappingURL=AccountNode.js.map