"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertErrorNode = exports.isErrorNode = exports.ErrorNode = void 0;
const utils_1 = require("../utils");
class ErrorNode {
    constructor(metadata, code, message) {
        this.nodeClass = 'ErrorNode';
        this.metadata = { ...metadata, name: (0, utils_1.mainCase)(metadata.name) };
        this.code = code;
        this.message = message;
    }
    static fromIdl(idl) {
        var _a, _b, _c, _d;
        const name = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
        const code = (_b = idl.code) !== null && _b !== void 0 ? _b : -1;
        const message = (_c = idl.msg) !== null && _c !== void 0 ? _c : '';
        const defaultDocs = [`${name}: '${message}'`];
        const docs = (_d = idl.docs) !== null && _d !== void 0 ? _d : defaultDocs;
        return new ErrorNode({ name, idlName: name, docs }, code, message);
    }
    accept(visitor) {
        return visitor.visitError(this);
    }
    get name() {
        return this.metadata.name;
    }
    get docs() {
        return this.metadata.docs;
    }
}
exports.ErrorNode = ErrorNode;
function isErrorNode(node) {
    return !!node && node.nodeClass === 'ErrorNode';
}
exports.isErrorNode = isErrorNode;
function assertErrorNode(node) {
    var _a;
    if (!isErrorNode(node)) {
        throw new Error(`Expected ErrorNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertErrorNode = assertErrorNode;
//# sourceMappingURL=ErrorNode.js.map