"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeEnumNode = exports.isTypeEnumNode = exports.TypeEnumNode = void 0;
const utils_1 = require("../utils");
const TypeEnumEmptyVariantNode_1 = require("./TypeEnumEmptyVariantNode");
const TypeEnumStructVariantNode_1 = require("./TypeEnumStructVariantNode");
const TypeEnumTupleVariantNode_1 = require("./TypeEnumTupleVariantNode");
class TypeEnumNode {
    constructor(name, variants) {
        this.nodeClass = 'TypeEnumNode';
        this.name = (0, utils_1.mainCase)(name);
        this.variants = variants;
    }
    static fromIdl(idl) {
        var _a;
        const variants = idl.variants.map((variant) => {
            if (!variant.fields || variant.fields.length <= 0) {
                return TypeEnumEmptyVariantNode_1.TypeEnumEmptyVariantNode.fromIdl(variant);
            }
            if (isStructField(variant.fields[0])) {
                return TypeEnumStructVariantNode_1.TypeEnumStructVariantNode.fromIdl(variant);
            }
            return TypeEnumTupleVariantNode_1.TypeEnumTupleVariantNode.fromIdl(variant);
        });
        return new TypeEnumNode((_a = idl.name) !== null && _a !== void 0 ? _a : '', variants);
    }
    accept(visitor) {
        return visitor.visitTypeEnum(this);
    }
    isScalarEnum() {
        return this.variants.every((variant) => variant.nodeClass === 'TypeEnumEmptyVariantNode');
    }
    isDataEnum() {
        return !this.isScalarEnum();
    }
}
exports.TypeEnumNode = TypeEnumNode;
function isTypeEnumNode(node) {
    return !!node && node.nodeClass === 'TypeEnumNode';
}
exports.isTypeEnumNode = isTypeEnumNode;
function assertTypeEnumNode(node) {
    var _a;
    if (!isTypeEnumNode(node)) {
        throw new Error(`Expected TypeEnumNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeEnumNode = assertTypeEnumNode;
function isStructField(field) {
    return typeof field === 'object' && 'name' in field && 'type' in field;
}
//# sourceMappingURL=TypeEnumNode.js.map