"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeMapNode = exports.isTypeMapNode = exports.TypeMapNode = void 0;
const TypeNode_1 = require("./TypeNode");
const TypeNumberNode_1 = require("./TypeNumberNode");
class TypeMapNode {
    constructor(key, value, options = {}) {
        var _a, _b;
        this.nodeClass = 'TypeMapNode';
        this.key = key;
        this.value = value;
        this.size = (_a = options.size) !== null && _a !== void 0 ? _a : {
            kind: 'prefixed',
            prefix: new TypeNumberNode_1.TypeNumberNode('u32'),
        };
        this.idlType = (_b = options.idlType) !== null && _b !== void 0 ? _b : 'hashMap';
    }
    static fromIdl(idl) {
        const idlType = 'hashMap' in idl ? 'hashMap' : 'bTreeMap';
        const [key, value] = 'hashMap' in idl ? idl.hashMap : idl.bTreeMap;
        let size;
        if (idl.size === 'remainder') {
            size = { kind: 'remainder' };
        }
        else if (typeof idl.size === 'number') {
            size = { kind: 'fixed', size: idl.size };
        }
        else if (idl.size) {
            size = { kind: 'prefixed', prefix: new TypeNumberNode_1.TypeNumberNode(idl.size) };
        }
        return new TypeMapNode((0, TypeNode_1.createTypeNodeFromIdl)(key), (0, TypeNode_1.createTypeNodeFromIdl)(value), { idlType, size });
    }
    accept(visitor) {
        return visitor.visitTypeMap(this);
    }
}
exports.TypeMapNode = TypeMapNode;
function isTypeMapNode(node) {
    return !!node && node.nodeClass === 'TypeMapNode';
}
exports.isTypeMapNode = isTypeMapNode;
function assertTypeMapNode(node) {
    var _a;
    if (!isTypeMapNode(node)) {
        throw new Error(`Expected TypeMapNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeMapNode = assertTypeMapNode;
//# sourceMappingURL=TypeMapNode.js.map