"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetJavaScriptRenderMapVisitor = void 0;
const prettier_1 = require("prettier");
const logs_1 = require("../../logs");
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const visitors_1 = require("../../visitors");
const RenderMap_1 = require("../RenderMap");
const utils_2 = require("../utils");
const GetJavaScriptTypeManifestVisitor_1 = require("./GetJavaScriptTypeManifestVisitor");
const JavaScriptImportMap_1 = require("./JavaScriptImportMap");
const RenderJavaScriptValueNode_1 = require("./RenderJavaScriptValueNode");
const DEFAULT_PRETTIER_OPTIONS = {
    semi: true,
    singleQuote: true,
    trailingComma: 'es5',
    useTabs: false,
    tabWidth: 2,
    arrowParens: 'always',
    printWidth: 80,
    parser: 'typescript',
};
class GetJavaScriptRenderMapVisitor extends visitors_1.BaseThrowVisitor {
    constructor(options = {}) {
        var _a, _b;
        super();
        this.program = null;
        this.options = {
            formatCode: (_a = options.formatCode) !== null && _a !== void 0 ? _a : true,
            prettierOptions: {
                ...DEFAULT_PRETTIER_OPTIONS,
                ...options.prettierOptions,
            },
            dependencyMap: {
                generated: '..',
                hooked: '../../hooked',
                core: '@metaplex-foundation/umi',
                mplEssentials: '@metaplex-foundation/mpl-essentials',
                ...options.dependencyMap,
                // Custom relative dependencies to link generated files together.
                generatedAccounts: '../accounts',
                generatedErrors: '../errors',
                generatedTypes: '../types',
            },
            typeManifestVisitor: (_b = options.typeManifestVisitor) !== null && _b !== void 0 ? _b : new GetJavaScriptTypeManifestVisitor_1.GetJavaScriptTypeManifestVisitor(),
        };
        this.resolvedInstructionAccountVisitor =
            new visitors_1.GetResolvedInstructionAccountsVisitor();
    }
    visitRoot(root) {
        const programsToExport = root.programs.filter((p) => !p.metadata.internal);
        const accountsToExport = root.allAccounts.filter((a) => !a.metadata.internal);
        const instructionsToExport = root.allInstructionsWithSubs.filter((i) => !i.metadata.internal);
        const definedTypesToExport = root.allDefinedTypes.filter((t) => !t.metadata.internal);
        const hasAnythingToExport = programsToExport.length > 0 ||
            accountsToExport.length > 0 ||
            instructionsToExport.length > 0 ||
            definedTypesToExport.length > 0;
        const ctx = {
            root,
            programsToExport,
            accountsToExport,
            instructionsToExport,
            definedTypesToExport,
            hasAnythingToExport,
        };
        const map = new RenderMap_1.RenderMap();
        if (programsToExport.length > 0) {
            map
                .add('programs/index.ts', this.render('programsIndex.njk', ctx))
                .add('errors/index.ts', this.render('errorsIndex.njk', ctx));
        }
        if (accountsToExport.length > 0) {
            map.add('accounts/index.ts', this.render('accountsIndex.njk', ctx));
        }
        if (instructionsToExport.length > 0) {
            map.add('instructions/index.ts', this.render('instructionsIndex.njk', ctx));
        }
        if (definedTypesToExport.length > 0) {
            map.add('types/index.ts', this.render('definedTypesIndex.njk', ctx));
        }
        return map
            .add('index.ts', this.render('rootIndex.njk', ctx))
            .mergeWith(...root.programs.map((program) => program.accept(this)));
    }
    visitProgram(program) {
        this.program = program;
        const { name } = program.metadata;
        const pascalCaseName = (0, utils_1.pascalCase)(name);
        const renderMap = new RenderMap_1.RenderMap()
            .mergeWith(...program.accounts.map((account) => account.accept(this)))
            .mergeWith(...program.definedTypes.map((type) => type.accept(this)));
        // Internal programs are support programs that
        // were added to fill missing types or accounts.
        // They don't need to render anything else.
        if (program.metadata.internal) {
            this.program = null;
            return renderMap;
        }
        renderMap
            .mergeWith(...program.instructionsWithSubs.map((ix) => ix.accept(this)))
            .add(`errors/${(0, utils_1.camelCase)(name)}.ts`, this.render('errorsPage.njk', {
            imports: new JavaScriptImportMap_1.JavaScriptImportMap()
                .add('core', ['ProgramError', 'Program'])
                .toString(this.options.dependencyMap),
            program,
            errors: program.errors.map((error) => ({
                ...error,
                prefixedName: (0, utils_1.pascalCase)(program.metadata.prefix) +
                    (0, utils_1.pascalCase)(error.metadata.name),
            })),
        }))
            .add(`programs/${(0, utils_1.camelCase)(name)}.ts`, this.render('programsPage.njk', {
            imports: new JavaScriptImportMap_1.JavaScriptImportMap()
                .add('core', ['Program', 'publicKey'])
                .add('errors', [
                `get${pascalCaseName}ErrorFromCode`,
                `get${pascalCaseName}ErrorFromName`,
            ])
                .toString(this.options.dependencyMap),
            program,
        }));
        this.program = null;
        return renderMap;
    }
    visitAccount(account) {
        var _a, _b;
        const typeManifest = account.accept(this.typeManifestVisitor);
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap()
            .mergeWith(typeManifest.imports)
            .add('core', [
            'Account',
            'assertAccountExists',
            'Context',
            'deserializeAccount',
            'PublicKey',
            'RpcAccount',
            'RpcGetAccountOptions',
            'RpcGetAccountsOptions',
            ...(!nodes.isTypeDefinedLinkNode(account.type) ? ['Serializer'] : []),
        ])
            .remove('generatedTypes', [account.name]);
        // Discriminator.
        const { discriminator } = account.metadata;
        let resolvedDiscriminator = null;
        if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'field' && discriminator.value !== null) {
            const rendered = (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(discriminator.value);
            imports.mergeWith(rendered.imports);
            resolvedDiscriminator = {
                kind: 'field',
                name: discriminator.name,
                value: rendered.render,
            };
        }
        else if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'field' &&
            !nodes.isTypeDefinedLinkNode(account.type)) {
            const discriminatorField = (_a = account.type.fields.find((f) => f.name === discriminator.name)) !== null && _a !== void 0 ? _a : null;
            const discriminatorValue = ((_b = discriminatorField === null || discriminatorField === void 0 ? void 0 : discriminatorField.metadata.defaultsTo) === null || _b === void 0 ? void 0 : _b.value)
                ? (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(discriminatorField.metadata.defaultsTo.value)
                : undefined;
            if (discriminatorValue) {
                imports.mergeWith(discriminatorValue.imports);
                resolvedDiscriminator = {
                    kind: 'field',
                    name: discriminator.name,
                    value: discriminatorValue.render,
                };
            }
        }
        else if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'size') {
            resolvedDiscriminator =
                account.metadata.size !== null
                    ? { kind: 'size', value: `${account.metadata.size}` }
                    : null;
        }
        // GPA Fields.
        const gpaFields = account.metadata.gpaFields.map((gpaField) => {
            const gpaFieldManifest = gpaField.type.accept(this.typeManifestVisitor);
            imports.mergeWith(gpaFieldManifest.imports);
            return { ...gpaField, manifest: gpaFieldManifest };
        });
        let resolvedGpaFields = null;
        if (gpaFields.length > 0) {
            imports.add('core', ['gpaBuilder']);
            resolvedGpaFields = {
                type: `{ ${gpaFields
                    .map((f) => `'${f.name}': ${f.manifest.looseType}`)
                    .join(', ')} }`,
                argument: `{ ${gpaFields
                    .map((f) => {
                    const offset = f.offset === null ? 'null' : `${f.offset}`;
                    return `'${f.name}': [${offset}, ${f.manifest.serializer}]`;
                })
                    .join(', ')} }`,
            };
        }
        // Seeds.
        const pdaHelperNeedsSerializer = account.metadata.seeds.filter((seed) => seed.kind !== 'programId')
            .length > 0;
        const seeds = account.metadata.seeds.map((seed) => {
            if (seed.kind !== 'variable')
                return seed;
            this.typeManifestVisitor.setImportStrategy('looseOnly');
            const seedManifest = seed.type.accept(this.typeManifestVisitor);
            this.typeManifestVisitor.setImportStrategy('all');
            imports.mergeWith(seedManifest.imports);
            return { ...seed, typeManifest: seedManifest };
        });
        if (seeds.length > 0) {
            imports.add('core', ['Pda']);
        }
        return new RenderMap_1.RenderMap().add(`accounts/${(0, utils_1.camelCase)(account.name)}.ts`, this.render('accountsPage.njk', {
            account,
            imports: imports.toString(this.options.dependencyMap),
            program: this.program,
            typeManifest,
            discriminator: resolvedDiscriminator,
            gpaFields: resolvedGpaFields,
            seeds,
            pdaHelperNeedsSerializer,
        }));
    }
    visitInstruction(instruction) {
        // Imports.
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap().add('core', [
            'AccountMeta',
            'Context',
            'PublicKey',
            'Signer',
            'WrappedInstruction',
        ]);
        // Accounts.
        const accounts = instruction
            .accept(this.resolvedInstructionAccountVisitor)
            .map((account) => {
            const hasDefaultValue = account.defaultsTo.kind !== 'none';
            if (account.defaultsTo.kind === 'pda') {
                const { seeds } = account.defaultsTo;
                Object.keys(seeds).forEach((seed) => {
                    const seedValue = seeds[seed];
                    if (seedValue.kind !== 'value')
                        return;
                    const seedManifest = (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(seedValue.value);
                    seedValue.render = seedManifest.render;
                    imports.mergeWith(seedManifest.imports);
                });
            }
            return {
                ...account,
                type: this.getInstructionAccountType(account),
                optionalSign: hasDefaultValue || account.isOptional ? '?' : '',
                hasDefaultValue,
            };
        });
        imports.mergeWith(this.getInstructionAccountImports(accounts));
        if (accounts.length > 0) {
            imports
                .add('core', 'checkForIsWritableOverride')
                .addAlias('core', 'checkForIsWritableOverride', 'isWritable');
        }
        // Arguments.
        const typeManifest = instruction.accept(this.typeManifestVisitor);
        imports.mergeWith(typeManifest.imports);
        if (!nodes.isTypeDefinedLinkNode(instruction.args) && instruction.hasData) {
            imports.add('core', ['Serializer']);
        }
        // Bytes created on chain.
        const bytes = instruction.metadata.bytesCreatedOnChain;
        if (bytes.kind !== 'none' && bytes.includeHeader) {
            imports.add('core', 'ACCOUNT_HEADER_SIZE');
        }
        if (bytes.kind === 'account') {
            const accountName = (0, utils_1.pascalCase)(bytes.name);
            const dependency = bytes.dependency === 'generated'
                ? 'generatedAccounts'
                : bytes.dependency;
            imports.add(dependency, `get${accountName}Size`);
        }
        // Remove imports from the same module.
        imports.remove('generatedTypes', [
            `${instruction.name}InstructionAccounts`,
            `${instruction.name}InstructionData`,
            `${instruction.name}InstructionDataArgs`,
        ]);
        // canMergeAccountsAndArgs
        let canMergeAccountsAndArgs = false;
        if (!nodes.isTypeDefinedLinkNode(instruction.args)) {
            const accountsAndArgsConflicts = this.getMergeConflictsForInstructionAccountsAndArgs(instruction);
            if (accountsAndArgsConflicts.length > 0) {
                (0, logs_1.logWarn)(`Accounts and args of instruction [${instruction.name}] have the following ` +
                    `conflicting attributes [${accountsAndArgsConflicts.join(', ')}]. ` +
                    `Thus, they could not be merged into a single input object. ` +
                    'You may want to rename the conflicting attributes.');
            }
            canMergeAccountsAndArgs = accountsAndArgsConflicts.length === 0;
        }
        return new RenderMap_1.RenderMap().add(`instructions/${(0, utils_1.camelCase)(instruction.name)}.ts`, this.render('instructionsPage.njk', {
            instruction,
            imports: imports.toString(this.options.dependencyMap),
            program: this.program,
            accounts,
            needsEddsa: accounts.some((a) => a.defaultsTo.kind === 'pda'),
            needsIdentity: accounts.some((a) => a.defaultsTo.kind === 'identity'),
            needsPayer: accounts.some((a) => a.defaultsTo.kind === 'payer'),
            typeManifest,
            canMergeAccountsAndArgs,
        }));
    }
    visitDefinedType(definedType) {
        const typeManifest = definedType.accept(this.typeManifestVisitor);
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap()
            .mergeWith(typeManifest.imports)
            .add('core', ['Context', 'Serializer'])
            .remove('generatedTypes', [definedType.name]);
        return new RenderMap_1.RenderMap().add(`types/${(0, utils_1.camelCase)(definedType.name)}.ts`, this.render('definedTypesPage.njk', {
            definedType,
            imports: imports.toString({
                ...this.options.dependencyMap,
                generatedTypes: '.',
            }),
            typeManifest,
        }));
    }
    get typeManifestVisitor() {
        return this.options.typeManifestVisitor;
    }
    getInstructionAccountType(account) {
        if (account.pdaBumpArg)
            return 'Pda';
        if (account.isOptionalSigner)
            return 'PublicKey | Signer';
        return account.isSigner ? 'Signer' : 'PublicKey';
    }
    getInstructionAccountImports(accounts) {
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap();
        accounts.forEach((account) => {
            if (account.pdaBumpArg) {
                imports.add('core', 'Pda');
            }
            if (account.defaultsTo.kind === 'publicKey') {
                imports.add('core', 'publicKey');
            }
            if (account.defaultsTo.kind === 'pda') {
                const pdaAccount = (0, utils_1.pascalCase)(account.defaultsTo.pdaAccount);
                const dependency = account.defaultsTo.dependency === 'generated'
                    ? 'generatedAccounts'
                    : account.defaultsTo.dependency;
                imports.add(dependency, `find${pdaAccount}Pda`);
                Object.values(account.defaultsTo.seeds).forEach((seed) => {
                    if (seed.kind === 'account') {
                        imports.add('core', 'publicKey');
                    }
                });
            }
            if (account.resolvedIsOptionalSigner) {
                imports.add('core', ['PublicKey', 'publicKey', 'Signer', 'isSigner']);
            }
            else if (account.resolvedIsSigner) {
                imports.add('core', 'Signer');
            }
            else {
                imports.add('core', 'PublicKey');
            }
        });
        return imports;
    }
    getMergeConflictsForInstructionAccountsAndArgs(instruction) {
        nodes.assertTypeStructNode(instruction.args);
        const allNames = [
            ...instruction.accounts.map((account) => account.name),
            ...instruction.args.fields.map((field) => field.name),
        ];
        const duplicates = allNames.filter((e, i, a) => a.indexOf(e) !== i);
        return [...new Set(duplicates)];
    }
    render(template, context, options) {
        const code = (0, utils_2.resolveTemplate)(`${__dirname}/templates`, template, context, options);
        return this.options.formatCode
            ? (0, prettier_1.format)(code, this.options.prettierOptions)
            : code;
    }
}
exports.GetJavaScriptRenderMapVisitor = GetJavaScriptRenderMapVisitor;
//# sourceMappingURL=GetJavaScriptRenderMapVisitor.js.map