"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseVoidVisitor = void 0;
class BaseVoidVisitor {
    visitRoot(root) {
        root.programs.forEach((program) => program.accept(this));
    }
    visitProgram(program) {
        program.accounts.forEach((account) => account.accept(this));
        program.instructions.forEach((instruction) => instruction.accept(this));
        program.definedTypes.forEach((type) => type.accept(this));
        program.errors.forEach((type) => type.accept(this));
    }
    visitAccount(account) {
        account.type.accept(this);
        account.variableSeeds.forEach((seed) => seed.type.accept(this));
        account.metadata.gpaFields.forEach((field) => field.type.accept(this));
    }
    visitInstruction(instruction) {
        instruction.args.accept(this);
        instruction.subInstructions.forEach((ix) => ix.accept(this));
    }
    visitDefinedType(definedType) {
        definedType.type.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitError(error) {
        //
    }
    visitTypeArray(typeArray) {
        typeArray.item.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeDefinedLink(typeDefinedLink) {
        //
    }
    visitTypeEnum(typeEnum) {
        typeEnum.variants.forEach((variant) => variant.accept(this));
    }
    visitTypeEnumEmptyVariant(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    typeEnumEmptyVariant) {
        //
    }
    visitTypeEnumStructVariant(typeEnumStructVariant) {
        typeEnumStructVariant.struct.accept(this);
    }
    visitTypeEnumTupleVariant(typeEnumTupleVariant) {
        typeEnumTupleVariant.tuple.accept(this);
    }
    visitTypeMap(typeMap) {
        typeMap.key.accept(this);
        typeMap.value.accept(this);
    }
    visitTypeOption(typeOption) {
        typeOption.item.accept(this);
    }
    visitTypeSet(typeSet) {
        typeSet.item.accept(this);
    }
    visitTypeStruct(typeStruct) {
        typeStruct.fields.forEach((field) => field.accept(this));
    }
    visitTypeStructField(typeStructField) {
        typeStructField.type.accept(this);
    }
    visitTypeTuple(typeTuple) {
        typeTuple.items.forEach((item) => item.accept(this));
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeBool(typeBool) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeBytes(typeBytes) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeNumber(typeNumber) {
        //
    }
    visitTypeNumberWrapper(typeNumberWrapper) {
        typeNumberWrapper.item.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypePublicKey(typePublicKey) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeString(typeString) {
        //
    }
}
exports.BaseVoidVisitor = BaseVoidVisitor;
//# sourceMappingURL=BaseVoidVisitor.js.map