"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toNodeSelectorFunction = void 0;
const utils_1 = require("../utils");
const toNodeSelectorFunction = (selector) => {
    if (typeof selector === 'function')
        return selector;
    const checkType = (node) => {
        if (!selector.type || selector.type === '*')
            return true;
        return selector.type === node.nodeClass;
    };
    const checkName = (node) => {
        if (selector.name === undefined)
            return true;
        return (0, utils_1.mainCase)(selector.name) === node.name;
    };
    const checkStack = (node, stack) => {
        if (selector.stack === undefined)
            return true;
        const selectorStack = Array.isArray(selector.stack)
            ? selector.stack
            : selector.stack.split('.');
        return stack.matchesWithNames(selectorStack);
    };
    const checkProgram = (node, stack, program) => {
        if (selector.program === undefined)
            return true;
        return !!program && (0, utils_1.mainCase)(selector.program) === program.name;
    };
    return (node, stack, program) => checkType(node, stack, program) &&
        checkName(node, stack, program) &&
        checkStack(node, stack, program) &&
        checkProgram(node, stack, program);
};
exports.toNodeSelectorFunction = toNodeSelectorFunction;
//# sourceMappingURL=NodeSelector.js.map