"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeStack = void 0;
const utils_1 = require("../utils");
class NodeStack {
    constructor(stack = []) {
        this.stack = [...stack];
    }
    push(node) {
        this.stack.push(node);
    }
    pop() {
        return this.stack.pop();
    }
    peek() {
        return this.isEmpty() ? undefined : this.stack[this.stack.length - 1];
    }
    all() {
        return [...this.stack];
    }
    isEmpty() {
        return this.stack.length === 0;
    }
    clone() {
        return new NodeStack(this.stack);
    }
    toString() {
        return this.toStringArray().join(' > ');
    }
    toStringArray() {
        return this.stack.map((node) => {
            switch (node.nodeClass) {
                case 'RootNode':
                    return 'Root';
                case 'ProgramNode':
                    return node.name ? `Program: ${node.name}` : 'Unnamed Program';
                case 'AccountNode':
                    return node.name ? `Account: ${node.name}` : 'Unnamed Account';
                case 'InstructionNode':
                    return node.name
                        ? `Instruction: ${node.name}`
                        : 'Unnamed Instruction';
                case 'DefinedTypeNode':
                    return node.name
                        ? `Defined Type: ${node.name}`
                        : 'Unnamed Defined Type';
                case 'ErrorNode':
                    return node.name ? `Error: ${node.name}` : 'Unnamed Error';
                case 'TypeEnumNode':
                    return node.name ? `Enum: ${node.name}` : 'Enum';
                case 'TypeEnumEmptyVariantNode':
                case 'TypeEnumStructVariantNode':
                case 'TypeEnumTupleVariantNode':
                    return node.name ? `Variant: ${node.name}` : 'Variant';
                case 'TypeNumberWrapperNode':
                    return `Number Wrapper: ${node.wrapper.kind}`;
                case 'TypeStructNode':
                    return node.name ? `Struct: ${node.name}` : 'Struct';
                case 'TypeStructFieldNode':
                    return node.name ? `Field: ${node.name}` : 'Field';
                default:
                    return (0, utils_1.titleCase)(node.nodeClass.replace(/Node$/, '').replace(/^Type/, ''));
            }
        });
    }
    matchesWithNames(names) {
        const remainingNames = [...names].map(utils_1.mainCase);
        this.stack.forEach((node) => {
            const nodeName = node.name;
            if (nodeName &&
                remainingNames.length > 0 &&
                remainingNames[0] === (0, utils_1.mainCase)(nodeName)) {
                remainingNames.shift();
            }
        });
        return remainingNames.length === 0;
    }
}
exports.NodeStack = NodeStack;
//# sourceMappingURL=NodeStack.js.map