"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidatorBag = void 0;
const logs_1 = require("../logs");
class ValidatorBag {
    constructor(items = []) {
        this.items = [];
        this.items = [...items];
    }
    add(item) {
        this.items.push(item);
        return this;
    }
    mergeWith(others) {
        others.forEach((other) => {
            this.items.push(...other.items);
        });
        return this;
    }
    error(message, node, stack) {
        return this.add({ message, level: 'error', node, stack: stack.clone() });
    }
    warn(message, node, stack) {
        return this.add({ message, level: 'warn', node, stack: stack.clone() });
    }
    info(message, node, stack) {
        return this.add({ message, level: 'info', node, stack: stack.clone() });
    }
    trace(message, node, stack) {
        return this.add({ message, level: 'trace', node, stack: stack.clone() });
    }
    debug(message, node, stack) {
        return this.add({ message, level: 'debug', node, stack: stack.clone() });
    }
    orderByLevel() {
        const orderedItems = this.items.sort((a, b) => (0, logs_1.getLevelIndex)(b.level) - (0, logs_1.getLevelIndex)(a.level));
        return new ValidatorBag(orderedItems);
    }
    log() {
        this.items.forEach((item) => {
            const hint = `Stack: ${item.stack.toString()}.`;
            switch (item.level) {
                case 'error':
                    (0, logs_1.logError)(item.message, hint);
                    break;
                case 'warn':
                    (0, logs_1.logWarn)(item.message, hint);
                    break;
                case 'info':
                    (0, logs_1.logInfo)(item.message, hint);
                    break;
                case 'trace':
                    (0, logs_1.logTrace)(item.message, hint);
                    break;
                case 'debug':
                default:
                    (0, logs_1.logDebug)(item.message, undefined, hint);
                    break;
            }
        });
    }
}
exports.ValidatorBag = ValidatorBag;
//# sourceMappingURL=ValidatorBag.js.map