"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoSetAnchorDiscriminatorsVisitor = void 0;
const sha256_1 = require("@noble/hashes/sha256");
const utils_1 = require("../../utils");
const nodes = __importStar(require("../../nodes"));
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
class AutoSetAnchorDiscriminatorsVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor() {
        super(...arguments);
        this.program = null;
    }
    visitProgram(program) {
        this.program = program;
        const visitedProgram = new nodes.ProgramNode(program.metadata, program.accounts
            .map((account) => account.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertAccountNode)), program.instructions
            .map((instruction) => instruction.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)), program.definedTypes, program.errors);
        this.program = null;
        return visitedProgram;
    }
    visitAccount(account) {
        var _a;
        const shouldAddDiscriminator = ((_a = this.program) === null || _a === void 0 ? void 0 : _a.metadata.origin) === 'anchor';
        if (!shouldAddDiscriminator)
            return account;
        if (nodes.isTypeDefinedLinkNode(account.type))
            return account;
        const idlName = (0, utils_1.snakeCase)(account.metadata.idlName);
        const hash = (0, sha256_1.sha256)(`global:${idlName}`).slice(0, 8);
        const discriminatorField = new nodes.TypeStructFieldNode({
            name: 'discriminator',
            docs: [],
            defaultsTo: {
                strategy: 'omitted',
                value: getValueNodeFromBytes(hash),
            },
        }, new nodes.TypeArrayNode(new nodes.TypeNumberNode('u8'), {
            size: { kind: 'fixed', size: 8 },
        }));
        return new nodes.AccountNode({
            ...account.metadata,
            discriminator: { kind: 'field', name: 'discriminator', value: null },
        }, new nodes.TypeStructNode(account.type.name, [
            discriminatorField,
            ...account.type.fields,
        ]));
    }
    visitInstruction(instruction) {
        var _a;
        const shouldAddDiscriminator = ((_a = this.program) === null || _a === void 0 ? void 0 : _a.metadata.origin) === 'anchor';
        if (!shouldAddDiscriminator)
            return instruction;
        if (nodes.isTypeDefinedLinkNode(instruction.args))
            return instruction;
        const idlName = (0, utils_1.snakeCase)(instruction.metadata.idlName);
        const hash = (0, sha256_1.sha256)(`global:${idlName}`).slice(0, 8);
        const discriminatorField = new nodes.TypeStructFieldNode({
            name: 'discriminator',
            docs: [],
            defaultsTo: {
                strategy: 'omitted',
                value: getValueNodeFromBytes(hash),
            },
        }, new nodes.TypeArrayNode(new nodes.TypeNumberNode('u8'), {
            size: { kind: 'fixed', size: 8 },
        }));
        return new nodes.InstructionNode(instruction.metadata, instruction.accounts, new nodes.TypeStructNode(instruction.args.name, [
            discriminatorField,
            ...instruction.args.fields,
        ]), instruction.subInstructions);
    }
}
exports.AutoSetAnchorDiscriminatorsVisitor = AutoSetAnchorDiscriminatorsVisitor;
function getValueNodeFromBytes(bytes) {
    return nodes.vList([...bytes].map((byte) => nodes.vScalar(byte)));
}
//# sourceMappingURL=AutoSetAnchorDiscriminatorsVisitor.js.map