"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.flattenStruct = exports.FlattenStructVisitor = void 0;
const logs_1 = require("../../logs");
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const TransformNodesVisitor_1 = require("./TransformNodesVisitor");
class FlattenStructVisitor extends TransformNodesVisitor_1.TransformNodesVisitor {
    constructor(map) {
        const transforms = Object.entries(map).map(([selectorStack, options]) => {
            const stack = selectorStack.split('.');
            const name = stack.pop();
            return {
                selector: { type: 'TypeStructNode', stack, name },
                transformer: (node) => (0, exports.flattenStruct)(node, options),
            };
        });
        super(transforms);
        this.map = map;
    }
}
exports.FlattenStructVisitor = FlattenStructVisitor;
const flattenStruct = (node, options = '*') => {
    nodes.assertTypeStructNode(node);
    const camelCaseOptions = options === '*' ? options : options.map(utils_1.camelCase);
    const shouldInline = (field) => options === '*' || camelCaseOptions.includes((0, utils_1.camelCase)(field.name));
    const inlinedFields = node.fields.reduce((all, one) => {
        if (nodes.isTypeStructNode(one.type) && shouldInline(one)) {
            all.push(...one.type.fields);
        }
        else {
            all.push(one);
        }
        return all;
    }, []);
    const inlinedFieldsNames = inlinedFields.map((arg) => arg.name);
    const duplicates = inlinedFieldsNames.filter((e, i, a) => a.indexOf(e) !== i);
    const uniqueDuplicates = [...new Set(duplicates)];
    const hasConflictingNames = uniqueDuplicates.length > 0;
    if (hasConflictingNames) {
        (0, logs_1.logWarn)(`Cound not flatten the attributes of struct [${node.name}] ` +
            `since this would cause the following attributes ` +
            `to conflict [${uniqueDuplicates.join(', ')}].` +
            'You may want to rename the conflicting attributes.');
    }
    return hasConflictingNames
        ? node
        : new nodes.TypeStructNode(node.name, inlinedFields);
};
exports.flattenStruct = flattenStruct;
//# sourceMappingURL=FlattenStructVisitor.js.map