"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformNodesVisitor = void 0;
const BaseNodeOrNullVisitor_1 = require("../BaseNodeOrNullVisitor");
const NodeSelector_1 = require("../NodeSelector");
const NodeStack_1 = require("../NodeStack");
class TransformNodesVisitor extends BaseNodeOrNullVisitor_1.BaseNodeOrNullVisitor {
    constructor(transforms) {
        super();
        this.stack = new NodeStack_1.NodeStack();
        this.program = null;
        this.transforms = transforms.map((transform) => ({
            ...transform,
            selector: (0, NodeSelector_1.toNodeSelectorFunction)(transform.selector),
        }));
    }
    visitRoot(root) {
        this.stack.push(root);
        const visitedRoot = super.visitRoot(root);
        this.stack.pop();
        return this.applyTransforms(visitedRoot);
    }
    visitProgram(program) {
        this.stack.push(program);
        this.program = program;
        const visitedProgram = super.visitProgram(program);
        this.program = null;
        this.stack.pop();
        return this.applyTransforms(visitedProgram);
    }
    visitAccount(account) {
        this.stack.push(account);
        const visitedAccount = super.visitAccount(account);
        this.stack.pop();
        return this.applyTransforms(visitedAccount);
    }
    visitInstruction(instruction) {
        this.stack.push(instruction);
        const visitedInstruction = super.visitInstruction(instruction);
        this.stack.pop();
        return this.applyTransforms(visitedInstruction);
    }
    visitDefinedType(definedType) {
        this.stack.push(definedType);
        const visitedDefinedType = super.visitDefinedType(definedType);
        this.stack.pop();
        return this.applyTransforms(visitedDefinedType);
    }
    visitError(error) {
        this.stack.push(error);
        const visitedError = super.visitError(error);
        this.stack.pop();
        return this.applyTransforms(visitedError);
    }
    visitTypeArray(typeArray) {
        this.stack.push(typeArray);
        const visitedTypeArray = super.visitTypeArray(typeArray);
        this.stack.pop();
        return this.applyTransforms(visitedTypeArray);
    }
    visitTypeDefinedLink(typeDefinedLink) {
        this.stack.push(typeDefinedLink);
        const visitedTypeDefinedLink = super.visitTypeDefinedLink(typeDefinedLink);
        this.stack.pop();
        return this.applyTransforms(visitedTypeDefinedLink);
    }
    visitTypeEnum(typeEnum) {
        this.stack.push(typeEnum);
        const visitedTypeEnum = super.visitTypeEnum(typeEnum);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnum);
    }
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant) {
        this.stack.push(typeEnumEmptyVariant);
        const visitedTypeEnumEmptyVariant = super.visitTypeEnumEmptyVariant(typeEnumEmptyVariant);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumEmptyVariant);
    }
    visitTypeEnumStructVariant(typeEnumStructVariant) {
        this.stack.push(typeEnumStructVariant);
        const visitedTypeEnumStructVariant = super.visitTypeEnumStructVariant(typeEnumStructVariant);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumStructVariant);
    }
    visitTypeEnumTupleVariant(typeEnumTupleVariant) {
        this.stack.push(typeEnumTupleVariant);
        const visitedTypeEnumTupleVariant = super.visitTypeEnumTupleVariant(typeEnumTupleVariant);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumTupleVariant);
    }
    visitTypeMap(typeMap) {
        this.stack.push(typeMap);
        const visitedTypeMap = super.visitTypeMap(typeMap);
        this.stack.pop();
        return this.applyTransforms(visitedTypeMap);
    }
    visitTypeOption(typeOption) {
        this.stack.push(typeOption);
        const visitedTypeOption = super.visitTypeOption(typeOption);
        this.stack.pop();
        return this.applyTransforms(visitedTypeOption);
    }
    visitTypeSet(typeSet) {
        this.stack.push(typeSet);
        const visitedTypeSet = super.visitTypeSet(typeSet);
        this.stack.pop();
        return this.applyTransforms(visitedTypeSet);
    }
    visitTypeStruct(typeStruct) {
        this.stack.push(typeStruct);
        const visitedTypeStruct = super.visitTypeStruct(typeStruct);
        this.stack.pop();
        return this.applyTransforms(visitedTypeStruct);
    }
    visitTypeStructField(typeStructField) {
        this.stack.push(typeStructField);
        const visitedTypeStructField = super.visitTypeStructField(typeStructField);
        this.stack.pop();
        return this.applyTransforms(visitedTypeStructField);
    }
    visitTypeTuple(typeTuple) {
        this.stack.push(typeTuple);
        const visitedTypeTuple = super.visitTypeTuple(typeTuple);
        this.stack.pop();
        return this.applyTransforms(visitedTypeTuple);
    }
    visitTypeBool(typeBool) {
        this.stack.push(typeBool);
        const visitedTypeBool = super.visitTypeBool(typeBool);
        this.stack.pop();
        return this.applyTransforms(visitedTypeBool);
    }
    visitTypeBytes(typeBytes) {
        this.stack.push(typeBytes);
        const visitedTypeBytes = super.visitTypeBytes(typeBytes);
        this.stack.pop();
        return this.applyTransforms(visitedTypeBytes);
    }
    visitTypeNumber(typeNumber) {
        this.stack.push(typeNumber);
        const visitedTypeNumber = super.visitTypeNumber(typeNumber);
        this.stack.pop();
        return this.applyTransforms(visitedTypeNumber);
    }
    visitTypeNumberWrapper(typeNumberWrapper) {
        this.stack.push(typeNumberWrapper);
        const visitedTypeNumberWrapper = super.visitTypeNumberWrapper(typeNumberWrapper);
        this.stack.pop();
        return this.applyTransforms(visitedTypeNumberWrapper);
    }
    visitTypePublicKey(typePublicKey) {
        this.stack.push(typePublicKey);
        const visitedTypePublicKey = super.visitTypePublicKey(typePublicKey);
        this.stack.pop();
        return this.applyTransforms(visitedTypePublicKey);
    }
    visitTypeString(typeString) {
        this.stack.push(typeString);
        const visitedTypeString = super.visitTypeString(typeString);
        this.stack.pop();
        return this.applyTransforms(visitedTypeString);
    }
    applyTransforms(node) {
        if (node === null)
            return null;
        const stack = this.stack.clone();
        const { program } = this;
        return this.transforms
            .filter(({ selector }) => selector(node, stack, program))
            .reduce((acc, { transformer }) => acc === null ? null : transformer(acc, stack, program), node);
    }
}
exports.TransformNodesVisitor = TransformNodesVisitor;
//# sourceMappingURL=TransformNodesVisitor.js.map