"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UseCustomAccountSerializerVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
class UseCustomAccountSerializerVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor(map) {
        super();
        this.map = Object.entries(map).reduce((acc, [selector, options]) => ({
            ...acc,
            [(0, utils_1.mainCase)(selector)]: parseLink(selector, options),
        }), {});
    }
    visitProgram(program) {
        const newDefinedTypes = program.definedTypes;
        program.accounts.forEach((account) => {
            var _a;
            const options = (_a = this.map[account.name]) !== null && _a !== void 0 ? _a : null;
            if (!options || !options.extract)
                return;
            if (nodes.isTypeDefinedLinkNode(account.type))
                return;
            const newType = new nodes.DefinedTypeNode({
                name: options.extractAs,
                idlName: account.metadata.idlName,
                docs: account.metadata.docs,
                internal: options.extractedTypeShouldBeInternal,
            }, account.type);
            newDefinedTypes.push(newType);
        });
        return new nodes.ProgramNode(program.metadata, program.accounts
            .map((account) => account.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertAccountNode)), program.instructions, newDefinedTypes, program.errors);
    }
    visitAccount(account) {
        var _a, _b, _c, _d;
        const options = (_a = this.map[account.name]) !== null && _a !== void 0 ? _a : null;
        if (!options)
            return account;
        if (nodes.isTypeDefinedLinkNode(account.type))
            return account;
        let newMetadata = account.metadata;
        // Ensure the discriminator value is not lost.
        if (((_b = account.metadata.discriminator) === null || _b === void 0 ? void 0 : _b.kind) === 'field' &&
            account.metadata.discriminator.value === null) {
            const fieldName = account.metadata.discriminator.name;
            const discriminatorField = (_c = account.type.fields.find((field) => field.name === fieldName)) !== null && _c !== void 0 ? _c : null;
            const discriminatorValue = (_d = discriminatorField === null || discriminatorField === void 0 ? void 0 : discriminatorField.metadata.defaultsTo) === null || _d === void 0 ? void 0 : _d.value;
            newMetadata = {
                ...account.metadata,
                discriminator: {
                    ...account.metadata.discriminator,
                    value: discriminatorValue !== null && discriminatorValue !== void 0 ? discriminatorValue : null,
                },
            };
        }
        return new nodes.AccountNode(newMetadata, new nodes.TypeDefinedLinkNode(options.name, {
            dependency: options.dependency,
        }));
    }
}
exports.UseCustomAccountSerializerVisitor = UseCustomAccountSerializerVisitor;
function parseLink(name, link) {
    const defaultOptions = {
        name: `${name}AccountData`,
        dependency: 'hooked',
        extract: false,
        extractAs: `${name}AccountData`,
        extractedTypeShouldBeInternal: true,
    };
    const options = typeof link === 'boolean' ? defaultOptions : { ...defaultOptions, ...link };
    return {
        ...options,
        name: (0, utils_1.mainCase)(options.name),
        extractAs: (0, utils_1.mainCase)(options.extractAs),
    };
}
//# sourceMappingURL=UseCustomAccountSerializerVisitor.js.map