import type { ConfigureOptions } from 'nunjucks';
import { Options as PrettierOptions } from 'prettier';
import * as nodes from '../../nodes';
import { Visitor, BaseThrowVisitor, ResolvedInstructionAccount, Dependency } from '../../visitors';
import { RenderMap } from '../RenderMap';
import { JavaScriptTypeManifest } from './GetJavaScriptTypeManifestVisitor';
import { JavaScriptImportMap } from './JavaScriptImportMap';
export type GetJavaScriptRenderMapOptions = {
    formatCode?: boolean;
    prettierOptions?: PrettierOptions;
    dependencyMap?: Record<Dependency, string>;
    typeManifestVisitor?: Visitor<JavaScriptTypeManifest> & {
        setImportStrategy: (importStrategy: 'all' | 'looseOnly' | 'strictOnly') => void;
    };
};
export declare class GetJavaScriptRenderMapVisitor extends BaseThrowVisitor<RenderMap> {
    readonly options: Required<GetJavaScriptRenderMapOptions>;
    private program;
    private resolvedInstructionAccountVisitor;
    constructor(options?: GetJavaScriptRenderMapOptions);
    visitRoot(root: nodes.RootNode): RenderMap;
    visitProgram(program: nodes.ProgramNode): RenderMap;
    visitAccount(account: nodes.AccountNode): RenderMap;
    visitInstruction(instruction: nodes.InstructionNode): RenderMap;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RenderMap;
    get typeManifestVisitor(): Visitor<JavaScriptTypeManifest> & {
        setImportStrategy: (importStrategy: "all" | "looseOnly" | "strictOnly") => void;
    };
    protected getInstructionAccountType(account: ResolvedInstructionAccount): string;
    protected getInstructionAccountImports(accounts: ResolvedInstructionAccount[]): JavaScriptImportMap;
    protected getMergeConflictsForInstructionAccountsAndArgs(instruction: nodes.InstructionNode): string[];
    protected render(template: string, context?: object, options?: ConfigureOptions): string;
}
