import * as nodes from '../../nodes';
import { Visitor } from '../../visitors';
import { JavaScriptImportMap } from './JavaScriptImportMap';
export type JavaScriptTypeManifest = {
    strictType: string;
    looseType: string;
    isEnum: boolean;
    serializer: string;
    imports: JavaScriptImportMap;
};
export declare class GetJavaScriptTypeManifestVisitor implements Visitor<JavaScriptTypeManifest> {
    readonly serializerVariable: string;
    private importStrategy;
    private definedName;
    constructor(serializerVariable?: string);
    setImportStrategy(strategy: GetJavaScriptTypeManifestVisitor['importStrategy']): void;
    visitRoot(): JavaScriptTypeManifest;
    visitProgram(): JavaScriptTypeManifest;
    visitAccount(account: nodes.AccountNode): JavaScriptTypeManifest;
    visitInstruction(instruction: nodes.InstructionNode): JavaScriptTypeManifest;
    visitDefinedType(definedType: nodes.DefinedTypeNode): JavaScriptTypeManifest;
    visitError(): JavaScriptTypeManifest;
    visitTypeArray(typeArray: nodes.TypeArrayNode): JavaScriptTypeManifest;
    visitTypeDefinedLink(typeDefinedLink: nodes.TypeDefinedLinkNode): JavaScriptTypeManifest;
    visitTypeEnum(typeEnum: nodes.TypeEnumNode): JavaScriptTypeManifest;
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant: nodes.TypeEnumEmptyVariantNode): JavaScriptTypeManifest;
    visitTypeEnumStructVariant(typeEnumStructVariant: nodes.TypeEnumStructVariantNode): JavaScriptTypeManifest;
    visitTypeEnumTupleVariant(typeEnumTupleVariant: nodes.TypeEnumTupleVariantNode): JavaScriptTypeManifest;
    visitTypeMap(typeMap: nodes.TypeMapNode): JavaScriptTypeManifest;
    visitTypeOption(typeOption: nodes.TypeOptionNode): JavaScriptTypeManifest;
    visitTypeSet(typeSet: nodes.TypeSetNode): JavaScriptTypeManifest;
    visitTypeStruct(typeStruct: nodes.TypeStructNode): JavaScriptTypeManifest;
    visitTypeStructField(typeStructField: nodes.TypeStructFieldNode): JavaScriptTypeManifest;
    visitTypeTuple(typeTuple: nodes.TypeTupleNode): JavaScriptTypeManifest;
    visitTypeBool(typeBool: nodes.TypeBoolNode): JavaScriptTypeManifest;
    visitTypeBytes(typeBytes: nodes.TypeBytesNode): JavaScriptTypeManifest;
    visitTypeNumber(typeNumber: nodes.TypeNumberNode): JavaScriptTypeManifest;
    visitTypeNumberWrapper(typeNumberWrapper: nodes.TypeNumberWrapperNode): JavaScriptTypeManifest;
    visitTypePublicKey(): JavaScriptTypeManifest;
    visitTypeString(typeString: nodes.TypeStringNode): JavaScriptTypeManifest;
    protected s(name: string): string;
    protected mergeManifests(manifests: JavaScriptTypeManifest[]): Pick<JavaScriptTypeManifest, 'imports' | 'isEnum'>;
    protected createDocblock(docs: string[]): string;
    protected getArrayLikeSizeOption(size: nodes.TypeArrayNode['size'], imports: JavaScriptImportMap): string | null;
}
